create or replace function binfs.clean ( istore varchar, imode varchar, istrict varchar, ifrom anyelement, ito anyelement ) returns integer as $$
-- imode: filetime, timestamp, integer, regexp
-- istrict: strict (strict to file), around (strict to folder)
declare
    ifp text; itp text;
    rt text;
    res integer;
begin
      select into rt row_to_json(r)::text from (select * from binfs.store where name = istore) r;
    if not found then
        perform core.log('binfs.clean','eror','store not found');
        return null;
    end if;
    imode := lower(trim(both ' ' from imode));
    istrict := lower(trim(both ' ' from istrict));
    if ifrom isnull then
        ifp := null;
    else    
        if 'timestamp' = imode or 'integer' = imode then
            select into ifp binfs.path(istore,ifrom);
            if ifp isnull then
                perform core.log('binfs.clean','eror','iconsistent store or from edge');
                return null;
            end if;    
        end if;    
    end if;
    if ito isnull then
        itp := null;
    else
        if 'timestamp' = imode or 'integer' = imode then
            select into itp binfs.path(istore,ito);
            if itp isnull then
                perform core.log('binfs.clean','eror','iconsistent store or to edge');
                return null;
            end if;    
        end if;    
    end if;    
    if imode isnull or not string_to_array(imode,'') <@ array['filetime','timestamp','integer','regexp'] then
        perform core.log('binfs.clean','eror','iconsistent mode');
        return null;
    end if;    
    if istrict isnull or not string_to_array(istrict,'') <@ array['strict','around'] then
        perform core.log('binfs.clean','eror','iconsistent strict');
        return null;
    end if;    
    select into res binfs._clean( rt, imode, istrict, ifrom::text, ito::text, ifp, itp );
    perform binfs._purge(rt);
    return res;
end;
$$ language plpgsql;


--
--
--

create or replace function binfs._clean ( istore text, imode text, istrict text, ifrom text, ito text, ifp text, itp text ) returns integer as $$
import json
import os
import shutil
import uuid
from datetime import datetime
from bsddb3 import db


def _count(ipath, idepth):
    if idepth < 0:
        return 1
    rv = plpy.execute("select binfs._statdir ( {0}, {1} ) ->> "
                      "'ocount' as count".format(plpy.quote_nullable(ipath),
                                                 idepth), 1)
    if rv.nrows() < 1:
        return None
    return int(rv[0]['count'])

ret = -1
param = {}
mdb = db.DB()
mdb.close()
metapath = ""

try:
    param = json.loads(istore)
    root = param['root']
    kind = param['class']
    opt = eval(param['opt'])
    mdep = len(opt)
except Exception as ex:
    plpy.execute("select core.log('binfs._clean',"
                 "'error','opt: '||{0})".format(plpy.quote_literal(str(ex))))
    return None

if 'filetime' == imode:
    """  don`t forget TODO it !!!! """
    pass
elif 'timestamp' == imode or 'integer' == imode:
    path_from = ifp
    path_to = itp
    cnt = 0
    if not ifp and not itp:
        try:
            cnt = _count(root, mdep)
            shutil.rmtree(root)
        except OSError as oe:
            plpy.execute("select core.log('binfs.clean','error',%s)" 
                         % (plpy.quote_literal(str(oe))))
            pass
    else:
        depth = -1
        while True:
            if (path_from and not path_to and path_from <= root) or (
                    path_to and not path_from and path_to <= root) or (
                    path_from and path_to and path_from == path_to):
                break
            if 'around' == istrict and depth < 0:
                cnt += 1
                depth += 1
                if path_from:
                    path_from = os.path.dirname(path_from)
                    try:
                        cnt += _count(path_from, depth)
                        shutil.rmtree(path_from)
                    except OSError as oe:
                        plpy.execute("select core.log('binfs.clean','error',%s)" 
                                     % (plpy.quote_literal(str(oe))))
                        pass
                    dir_from,name_from = os.path.split(path_from)
                    metapath = os.path.join(dir_from,".%s..meta"%(name_from))
                    try:
                        os.unlink(metapath)
                    except OSError as oe:
                        plpy.execute("select core.log('binfs.clean','error',%s)" 
                                     % (plpy.quote_literal(str(oe))))
                        pass
                if path_to:
                    path_to = os.path.dirname(path_to)
                    try:
                        cnt += _count(path_to, depth)
                        shutil.rmtree(path_to)
                    except OSError as oe:
                        plpy.execute("select core.log('binfs.clean','error',%s)" 
                                     % (plpy.quote_literal(str(oe))))
                        pass
                    dir_to,name_to = os.path.split(path_to)
                    metapath = os.path.join(dir_to,".%s..meta"%(name_to))
                    try:
                        os.unlink(metapath)
                    except OSError as oe:
                        plpy.execute("select core.log('binfs.clean','error',%s)" 
                                     % (plpy.quote_literal(str(oe))))
                        pass
            if path_from:
                dir_from, name_from = os.path.split(path_from)
            if path_to:
                dir_to, name_to = os.path.split(path_to)
            if path_from:
                if os.path.exists(dir_from):
                    has_meta = False
                    if depth < 0:
                        try:
                            mdir, mblock = os.path.split(dir_from)
                            mdb.close()
                            mdb = db.DB()
                            mdb.open(os.path.join(mdir,".%s..meta"%(mblock)), dbtype=db.DB_BTREE )
                            has_meta = True
                        except db.DBNoSuchFileError:
                            has_meta = False
                            pass
                    try:
                        for f in os.listdir(dir_from):
                            if f > name_from and not (path_to and
                                                      dir_to == dir_from and
                                                      f >= name_to):
                                pth = os.path.join(dir_from, f)
                                try:
                                    if os.path.isdir(pth):
                                        cnt += _count(pth, depth)
                                        shutil.rmtree(pth)
                                        metapath = os.path.join(dir_from,".%s..meta"%(f))
                                        os.unlink(metapath)
                                    else:
                                        if depth < 0:
                                            cnt += 1
                                            os.unlink(pth)
                                            if has_meta: 
                                                mdb.delete(f)
                                except OSError as oe:
                                    plpy.execute("select core.log('binfs.clean','error',%s)" 
                                                 % (plpy.quote_literal(str(oe))))
                                    continue
                                except db.DBNotFoundError: 
                                    continue    
                    except OSError as oe:
                        plpy.execute("select core.log('binfs.clean','error',%s)" 
                                     % (plpy.quote_literal(str(oe))))
                        pass
            if path_to:
                if os.path.exists(dir_to):
                    has_meta = False
                    if depth < 0:
                        try:
                            mdir, mblock = os.path.split(dir_to)
                            mdb.close()
                            mdb = db.DB()
                            mdb.open(os.path.join(mdir,".%s..meta"%(mblock)), dbtype=db.DB_BTREE )
                            has_meta = True
                        except db.DBNoSuchFileError:
                            has_meta = False
                            pass
                    try:
                        for f in os.listdir(dir_to):
                            if f < name_to and not (path_from and
                                                    dir_from == dir_to and
                                                    f <= name_from):
                                pth = os.path.join(dir_to, f)
                                try:
                                    if os.path.isdir(pth):
                                        cnt += _count(pth, depth)
                                        shutil.rmtree(pth)
                                        metapath = os.path.join(dir_to,".%s..meta"%(f))
                                        os.unlink(metapath)
                                    else:
                                        if depth < 0:
                                            cnt += 1
                                            os.unlink(pth)
                                            if has_meta: 
                                                mdb.delete(f)
                                except OSError as oe:
                                    plpy.execute("select core.log('binfs.clean','error',%s)" 
                                                 % (plpy.quote_literal(str(oe))))
                                    continue
                                except db.DBNotFoundError: 
                                    continue    
                    except OSError as oe:
                        pass
            if path_from:
                path_from = dir_from
            if path_to:
                path_to = dir_to
            depth += 1
        ret = cnt
elif 'regexp' == imode:
    """  don`t forget TODO it !!!! """
    pass
else:
    plpy.execute("select core.log('binfs._clean','error','invalid mode');")
return ret
$$ language plpython3u;

--
--
--
create or replace function binfs._purge ( istore text ) returns text[] as $$
def _dirproc(ipath, idepth):
    import os
    import shutil

    res = []
    if not os.path.exists(ipath):
        return res
    if idepth < 0:
        return res
    if idepth > 0:
        cnt = 0
        for f in os.listdir(ipath):
            if f.startswith('.'):
                continue
            if f.startswith('__binfs'):
                continue
            if os.path.isdir(os.path.join(ipath, f)):
                res += _dirproc(os.path.join(ipath, f), idepth - 1)
                if os.path.exists(os.path.join(ipath, f)):
                    cnt += 1
        if cnt == 0:
            shutil.rmtree(ipath, ignore_errors=True)
            res.append(ipath)
    else:
        st = os.lstat(ipath)
        sto = dict((name, getattr(st, name)) for name in dir(st) if
                   not name.startswith('__'))
        sth = "%x" % (hash(tuple(sorted(hash(x) for x in sto.items()))) &
                      0xffffffffffffffff)
        rescan = True
        pure = False
        idir, iname = os.path.split(ipath)
        stfile = os.path.join(idir, '.stat.' + iname + '.' + str(sth))
        if os.path.exists(stfile):
            try:
                with open(stfile) as stfd:
                    stf = json.load(stfd)
                if stf['stats']['ocount'] == 0:
                    pure = True
                    rescan = False
            except Exception as ex:
                plpy.execute("select core.log('binfs._purge','error',"
                             "'prescan: '||{0})"
                             .format(plpy.quote_literal(str(ex))))
        if rescan:
            if not os.listdir(ipath):
                pure = True
        try:
            if pure:
                shutil.rmtree(ipath, ignore_errors=True)
                res.append(ipath)
        except Exception as ex:
            plpy.execute("select core.log('binfs._purge',"
                         "'error','rmtree: '||{0})"
                         .format(plpy.quote_literal(str(ex))))
    return res

import json

param = {}
res = []
try:
    param = json.loads(istore)
    root = param['root']
    kind = param['class']
    opt = eval(param['opt'])
    depth = len(opt)
except Exception as ex:
    plpy.execute("select core.log('binfs._purge','error','opt: '||{0})"
                 .format(plpy.quote_literal(str(ex))))
    return None
res = _dirproc(root, depth)
return tuple(res)
$$ language plpython3u;

--
--
--
create or replace function binfs.rmtree( filepath text ) returns integer as $$
import os
import shutil

try:
    if not os.path.exists(filepath):
        return 0

    if os.path.isdir(filepath):
        shutil.rmtree(filepath)
    else:
        os.remove(filepath)
    return 1 
except Exception as ex:
    plpy.execute("select core.log('binfs.rmtree','error','exists: '||{0})"
                 .format(plpy.quote_nullable(str(ex))))
    return -1
$$ language plpython3u;

--
--
--
create or replace function binfs._remove ( istore text ) returns integer as $$
declare
    res integer;
begin    
    select into res binfs.rmtree(root) from binfs.store where name = istore;
    delete from binfs.store where name = istore;
    return res;
end;    
$$ language plpgsql;

