create or replace function binfs.realpath ( istore text, ikey text ) returns varchar as $$
import os
import json, uuid
from datetime import datetime

"""
    Opt format:
    -----------
    timestamp:
        array of strftime formats ['%Y','%M','....'], file is of ts float
    string:
        ['separator',['the{0}','a{1}']] - split separator and list of format strings
    bytes:
        [1,-1,2] - list of byte numbers starting with 1 (-1) - from beginning or end of hex string
        hex formed from decimal,hex or uuid
    integer:
        [10,[4,3,2,]] - low mask digits in dec
        [16,[4,3,2,]] - low mask bytes in hex
"""

path = None
param = {}
try:
    param = json.loads(istore)
    root = param['root']
    opt = eval(param['opt'])
except Exception as ex:
    plpy.execute("select core.log('binfs.realpath',"
                 "'error','opt: '||{0})".format(plpy.quote_literal(str(ex))))
    return None

if   'timestamp' == param['class']:
    dt = None
    if ikey.find('.') > 0:
        try:
            tf = float(ikey)
            dt = datetime.utcfromtimestamp(tf)
        except ValueError:    
            dt = None
    if dt is None:
        try:
            tf = long(ikey) / 1e6
            dt = datetime.utcfromtimestamp(tf)
        except ValueError:
            dt = None
    if dt is None:
        try:
            r = plpy.execute("select floor(extract(epoch from '{0}'::timestamptz)) + to_char('{0}'::timestamptz,'US')::integer/1000000.0 as \"epoch\"".format(ikey),1)
            tf = r[0]['epoch']
            dt = datetime.utcfromtimestamp(tf)
        except Exception as ex:
            plpy.execute("select core.log('binfs.realpath','error','timestamp: '||{0})".format(plpy.quote_literal(str(ex))))
            return None
    path = root
    for part in opt:
        path = os.path.join(path,dt.strftime(part))
    path = os.path.join(path,"{:.6f}".format(tf))

elif 'string' == param['class']:
    delim = opt[0]
    parts = [x.strip() for x in ikey.split(delim)]
    path = root
    for part in opt[1]:
        path = os.path.join(path, part.format(parts))
    path = os.path.join(path, ikey)

elif 'bytes' == param['class']:
    val = None
    try:
        val = "%X" % long(round(float(ikey)))
        if len(val) & 1 == 1:
            val = '0' + val
    except ValueError:
        val = None
    if not val:
        try:
            val = "%X" % long(ikey, 16)
            if len(val) & 1 == 1:
                val = '0' + val
        except ValueError:
            val = None
    if not val:
        try:
            val = uuid.UUID(ikey).hex.upper()
            if len(val) & 1 == 1:
                val = '0' + val
        except ValueError:
            val = None
    if not val:
        val = ikey
    path = root
    for i in opt:
        u = i * 2 - 1
        if u == 0:
            u = None
        l = i * 2 + 1
        if l == 0:
            l = None
        path = os.path.join(path, val[l:u])
    path = os.path.join(path, ikey)

elif 'integer' == param['class']:
    val = None
    try:
        val = long(val, 10)
    except ValueError:
        val = None
    if not val:
        try:
            val = long(ikey, 16)
        except ValueError:
            val = None
    if not val:
        plpy.execute("select core.log('binfs.realpath','error',"
                     "'invalid integer')")
        return None
    else:
        if opt[0] == 10:
            path = root
            for d in opt[1]:
                x = val - val % pow(10, d)
                path = os.path.join(path, "{0:020d}".format(x))
            path = os.path.join(path, "{0:020d}".format(val))
        elif opt[0] == 16:
            path = root
            for d in opt[1]:
                x = val >> d * 8 << d * 8
                path = os.path.join(path, "{0:016X}".format(x))
            path = os.path.join(path, "{0:016X}".format(val))
        else:
            plpy.execute("select core.log('binfs.realpath',"
                         "'error','invalid integer opt')")
            return None
else:
    plpy.execute("select core.log('binfs.realpath','error',"
                 "'Invalid key class')")
    return None

return path
$$ language plpython3u;
