CREATE or replace FUNCTION lib.getvolumefreespace(ipath text) RETURNS bigint LANGUAGE plpython3u AS $$
import ctypes
import os
import platform

path = os.path.abspath(ipath)
while not os.path.ismount(path):
    path = os.path.dirname(path)
if platform.system() == "Windows":
    free_bytes = ctypes.c_ulonglong(0)
    ctypes.windll.kernel32.GetDiskFreeSpaceExW(ctypes.c_wchar_p(path),
                                               None, None,
                                               ctypes.pointer(free_bytes))
    return free_bytes.value
else:
    svf = os.statvfs(path)
    return svf.f_frsize * svf.f_bavail
$$;


create or replace function lib._dirsize_posix( ipath text ) returns bigint as $$
import os

def _weigh(idir):
    sz = 0
    if not os.path.exists(idir):
        return 0
    for f in os.listdir(idir):
        fp = os.path.join(idir, f)
        if os.path.isdir(fp):
            sz += _weigh(fp)
        else:
            st = os.lstat(fp)
            sz += st.st_size
    return sz

return _weigh(ipath)
$$ language plpython3u;

create or replace function lib._dirsize_win32( ipath text ) returns bigint as $$
import win32com.client as com

fso = com.Dispatch("Scripting.FileSystemObject")
folder = fso.GetFolder(ipath)
return folder.Size
$$ language plpython3u;



CREATE or replace FUNCTION lib.getdirsize( ipath text DEFAULT '.'::text) RETURNS bigint AS $$
DECLARE
	res bigint;
	os varchar;
BEGIN
	os := _s('_os');
	if os = 'linux' then
		select into res lib._dirsize_posix( ipath );
	elsif os = 'windows' then	
		select into res lib._dirsize_win32( ipath );
	end if;	
	return res;
END;
$$ language plpgsql;


