CREATE OR REPLACE FUNCTION service.maintenance_clip(itag character varying) RETURNS void
    LANGUAGE plpgsql
    AS $_$
declare
	r record;
	tpl text;
	cmd text;
    _guid uuid;
	cnt integer;
begin
perform true from core.lock where name = 'maintenance.clip' for update nowait;
update core.lock set lastlock = clock_timestamp() where name = 'maintenance.clip';
	
	if itag = 'age' then
		begin
			cmd := '';
			for r in select * from service.maintenance_clipage for update nowait loop
				execute 'select ''delete from ''||$1||'' cascade where ''||$2||''::timestamptz < (clock_timestamp() - interval ''''''||$3||'''''') and ''||coalesce($4,''true'')||'';'';'
				into tpl
				using r.table,r.column,r.age,r.andcond;
				cmd := cmd || tpl || ' ';
			end loop;
			if cmd <> '' then
				execute cmd;
				get diagnostics cnt = ROW_COUNT;
				insert into service.maintenance_log ( key, value ) values ( E'CLIP_AGE', E'deleted: '||cnt::text );
			end if;	
		exception 
			when lock_not_available then
				insert into service.maintenance_log ( key, value ) values ( E'CLIP_AGE_LOCK', E'duplicate instance' );
			when others then
				insert into service.maintenance_log ( key, value ) values ( E'CLIP_AGE_ERROR', E'ERROR:{state:'||SQLSTATE||E',errm:'||SQLERRM||E'}' );
		end;	
	elsif itag = 'collection' then
        for _guid in select id from collection_event_origin where param->>'manual' isnull or not (param->>'manual')::boolean loop
            select into cnt 1 from public.eventindex where origin = _guid limit 1;
            if cnt isnull then
                delete from collection_event_lane where origin = _guid;
                delete from collection_event_origin where id = _guid;
				insert into service.maintenance_log ( key, value ) values ( E'CLIP_COLLECTION', E'origin deleted: '||_guid::text );
            end if;    
        end loop;
    end if;
	
	update core.lock set lastrelease = clock_timestamp() where name = 'maintenance.clip';

exception when lock_not_available then
	insert into service.maintenance_log ( key, value ) values ( E'CLIP_LOCK', E'duplicate instance' );
end;
$_$;

