#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# (k) made-on-the-knee-of /dragon, 2017

import os, sys, random, math
import json, uuid, hashlib
from datetime import datetime, timedelta, tzinfo
import time

#------------ timezones on the knee

class utczone(tzinfo):
    def utcoffset(self, dt):
        return timedelta(0)
    def dst(self,dt):
        return timedelta(0)
    def tzname(self, dt):
        return ('UTC','UTC')

class localzone(tzinfo):
    def utcoffset(self, dt):
        return timedelta(seconds=-time.timezone) + self.dst(dt)
    def dst(self, dt):
        return timedelta(hours=time.daylight)
    def tzname(self,dt):
        return time.tzname[time.daylight]

#------------ lists

l_type = [
     "transit"
    ,"transit"
    ,"transit"
    ,"transit"
    ,"transit"
    ,"transit"
    ,"transit"
    ,"transit"
    ,"transit"
    ,"transit"
    ,"transit"
    ,"transit"
    ,"transit"
    ,"red-light"
    ,"wrong-way"
    ,"parking"
    ,"overload"
    ,"oversize"
    ,"wanted"
    ,"hazard"
    ,"transit"
    ,"transit"
    ,"transit"
    ,"transit"
    ,"transit"
    ,"transit"
    ,"transit"
    ,"transit"
    ,"transit"
    ,"transit"
    ,"transit"
    ,"transit"
]

l_country = [
    804,
    860,
    643,
    784
]

l_region = {
    784: [1,2,3,4,5,6,7],
    804: [    5,    7,    9, 12,    14,    18,    21,    23,    26,    30,    32,    35,    40,    43,    46,    48,    51,    53,    56,    59,    61,    63,    65,    68,    71,    74,    77]
}

l_origin = [
    {
        "name":  "Source0"
        ,"uuid": "c5186a04-03f3-4d35-b93c-bf165a55c4fd"
        ,"time": "2017-09-12T12:46:47.037120"
        ,"location": { "place": "Place 001" }
    },
    {
        "name":  "Source0"
        ,"uuid": "4858b103-ade0-4de3-ab4f-4972dfabe521"
        ,"time": "2017-09-11T11:46:47.037120"
        ,"location": { "place": "Place 002" }
    },
    {
        "name":  "Source1"
        ,"uuid": "4f879af2-29a5-4d7b-9d76-0055e763258e"
        ,"time": "2017-09-10T10:46:47.037120"
        ,"location": { "place": "Place 003" }
    },
    {
        "name":  "SourceF"
        ,"uuid": "22ee04ef-5dbe-4595-a22d-e25c39a12c21"
        ,"time": "2017-09-09T09:46:47.037120"
        ,"location": { "place": "Place 004" }
    }
    ]


#------------ tools

def _rlist( l ):
    if not isinstance(l, list): return None
    random.shuffle(l)
    return random.choice(l)

def _rint( l, h = None ):
    if not h: h,l = l,0
    return l + int(math.floor(random.random()*(h-l)))

def _rtext( template ):
    s = ""
    for c in template:
        if c == 'X':
            s += chr(_rint(65,90))
        elif c == 'Z':
            s += chr(_rint(48,57))
        else:
            s += c
    return s        

def _scale ( val, scale ):
    if not val or not scale: return None
    try:
        return [s for s in scale if s < val][-1]
    except IndexError:
        return scale[0]

#------------ generator


def mkevent( ctime = datetime.now() ):
    slimit = 60
    smargin = 20
    _ptimelimit = _rint(300,3600)
    _ptime = _rint(_ptimelimit,_ptimelimit*2)
    e = {
        "uuid": str(uuid.uuid4()),
        "time": ctime.isoformat(),        
        "type": _rlist(l_type),
        "origin": _rlist(l_origin),
        "measurements": []
    }
    _t = e['type']
    for i in range(_rint(1,1)):
        _m = {
            "uuid": str(uuid.uuid4()) 
        }
        _v = {
                "license-plates": []
            }
        
        if e['origin']['name'] == 'SourceF':
            ss = _rint(20,100)
            d = 1234
            _v['section-speed'] = {
                "measured": ss,
                "name": "Section-F",
                "distance": d,
                "time":  int(ss * 3.6e6 / d)
            }    
            if ss > slimit+smargin:
                    _t = 'section-speed'; 
                    e['type'] = 'section-speed'    
        else:
            s = _rint(20,100)
            _v['spot-speed'] = {
                'radar': {"measured": s},
                'camera': {"measured":_rint(s-5,s+5)}
            }    
            if s > slimit+smargin:
                _t = 'spot-speed' 
                e['type'] = 'spot-speed'
        for j in range(_rint(1,2)):
            c = _rlist(l_country)
            _v['license-plates'].append(
                {
                    "type":{
                        "country": { "numeric": c, "subdivision": { "code": str(_rlist(l_region[c])) if c in l_region else None }},
                        "extra":{"autosdk": _rtext('ZZZZ')},
                    },
                    "text": {"ucode":_rtext('XXZZZZXX')},
                    "weight": _rint(50,100)
                }
            )
        _m['waypoints'] = {}
        _m['waypoints']['best'] = {
            "time": (ctime-timedelta(seconds=0.7)).isoformat(),
            "location": { "place": "place", "way": "way" },
            "geolocation": { "latitude": 50.441073, "longitude": 30.514813 },
            "lane": "lane-%s" % (_rint(3)),
            "dir": "approaching",
            "vehicle": _v
        }
        _m['waypoints']['best']['images'] = {
            "vehicle": { "name":"/images/vehicle/%s"%((ctime-datetime.fromtimestamp(0)).total_seconds()),"checksum":["md5",hashlib.md5(ctime.isoformat().encode('utf-8')).hexdigest()]},
            "vehicle_thumb": { "name":"/images/vehicle_thumb/%s"%((ctime-datetime.fromtimestamp(0)).total_seconds()),"checksum":["md5",hashlib.md5(ctime.isoformat().encode('utf-8')).hexdigest()]},
            "scene.measurement": { "name":"/images/scene.measurement/%s"%((ctime-datetime.fromtimestamp(0)).total_seconds()),"checksum":["md5",hashlib.md5(ctime.isoformat().encode('utf-8')).hexdigest()]}
        }
        e['measurements'].append(_m)    
    if _t == 'spot-speed':
        e['evidence']    = {
            "args": {
                "speed-limit": slimit,
                "max-overspeed": smargin,
                "lanes": ['lane-0', 'lane-1', 'lane-2'],
                "profile": {
                    "id": "spot_overspeed_" + str(_scale(e['measurements'][0]['waypoints']['best']['vehicle']['spot-speed']['radar']['measured'] - slimit,[20,40,60]))
                }
            },    
            "measurements":{
                "overspeed": e['measurements'][0]['waypoints']['best']['vehicle']['spot-speed']['radar']['measured'] - slimit     
            },
            "images": {
                "camera.speed.start": { "name":"/images/camera.speed.start/%s"%((ctime-datetime.fromtimestamp(0)).total_seconds()),"checksum":["md5",hashlib.md5(str(ctime).encode('utf-8')).hexdigest()]},
                "camera.speed.start": { "name":"/images/camera.speed.start/%s"%((ctime-datetime.fromtimestamp(0)).total_seconds()),"checksum":["md5",hashlib.md5(str(ctime).encode('utf-8')).hexdigest()]},

            }
        }        
    if _t == 'section-speed':
        e['evidence']    = {
            "args": {
                "speed-limit": slimit,
                "max-overspeed": smargin,
                "profile": {
                    "id": "section_overspeed_" + str(_scale(e['measurements'][0]['waypoints']['best']['vehicle']['section-speed']['measured'] - slimit,[20,40,60]))
                }
            },    
            "measurements":{
                "overspeed": e['measurements'][0]['waypoints']['best']['vehicle']['section-speed']['measured'] - slimit     
            },
            "images": {
                
            }
        }            
    if _t == "parking":
        e['evidence']    = {
            "args": {
                "time-limit": _ptimelimit,
                "profile": {
                    "id": "stop_overtime_" + str(_scale(_ptime,[300,1800])) 
                }
            },    
            "measurements":{
                "time": _ptime 
            },
            "images": {
                
            }
        }            
    if _t == "overload":
        e["evidence"] = {
            "args": {
                "total-weight-limit": _scale(_rint(1000,10000),[3000,5000,9000]),
                "profile": {
                    "id": "overload" 
                }
            },    
            "measurements":{
            }
        }
    if _t == 'wanted':
        e['evidence'] = {
            "args": {
                "list-id": "List 1",
                "list-origin": "hwp",
                "alarm-level": 3
            }
        }
    if _t == "red-light":
        e["evidence"] = {}
    if _t == "oversize":
        e["evidence"] = {}
    if _t == "hazard":
        e["evidence"] = {}
    return e    
    




